//
//  pCOS sample application for dumping PDF information
//
using System;
using System.Text;
using PLOP_dotnet;

public class Dumper
{
    static String docoptlist = "requiredmode=minimum";
    static String globaloptlist = "";

    /* This is where input files live. Adjust as necessary. */
    const string searchpath = "../data ../../data";


    static int Main(string[] args)
    {
        int count, recipientcount;
        int     i, doc;
        bool    plainmetadata;
        string  objtype, pcosmode, optlist, sigtype;
        PLOP plop = null;

        try
        {
            if (args.Length != 1)
            {
#if NET40
                Console.WriteLine("usage: dumper <filename>");
#else
                Console.WriteLine("usage: dotnet run <filename>");
#endif
                return(2);
            }

            plop = new PLOP();

            plop.set_option(globaloptlist);

            optlist = "searchpath={" + searchpath + "}";
            plop.set_option(optlist);

            if ((doc = plop.open_document(args.GetValue(0).ToString(), docoptlist)) == -1)
            {
                Console.WriteLine("ERROR " + plop.get_errmsg());
                return(2);
            }

            /* --------- general information (always available) */

            pcosmode = plop.pcos_get_string(doc, "pcosmodename");

            Console.WriteLine("       File name: " + plop.pcos_get_string(doc,"filename"));

            Console.WriteLine("     PDF version: " +
                plop.pcos_get_string(doc, "pdfversionstring"));

            Console.WriteLine("      Encryption: " +
                plop.pcos_get_string(doc, "encrypt/description"));

            recipientcount = (int) plop.pcos_get_number(doc, "length:encrypt/recipients");
            if (recipientcount > 0 ){
                Console.WriteLine("Recipient groups: " + recipientcount);
                Console.WriteLine("      Digital ID: ");

                if (pcosmode == "full" )
                    Console.WriteLine("supplied digital ID has master permissions");
                    else if ( pcosmode == "restricted")
                    Console.WriteLine("supplied digital ID doesn't have master permissions");
                else
                    Console.WriteLine("no suitable digital ID supplied");
                
            }

            Console.WriteLine("       Master pw: " +
                ((plop.pcos_get_number(doc, "encrypt/master") != 0) ? "yes" : "no"));

            Console.WriteLine("         User pw: " +
                ((plop.pcos_get_number(doc, "encrypt/user") != 0) ? "yes" : "no"));

            Console.WriteLine("    Text copying: " +
                ((plop.pcos_get_number(doc, "encrypt/nocopy") != 0) ? "no" : "yes"));

            Console.WriteLine("      Linearized: " +
                ((plop.pcos_get_number(doc, "linearized") != 0) ? "yes" : "no"));

            Console.WriteLine("");

            if (pcosmode == "minimum")
            {
                Console.WriteLine(
                    "Minimum mode: no more information available\n");
                return(0);
            }

            /* --------- more details (requires at least user password) */

            Console.WriteLine("    PDF/X status: " + plop.pcos_get_string(doc, "pdfx"));
            Console.WriteLine("    PDF/A status: " + plop.pcos_get_string(doc, "pdfa"));
            Console.WriteLine("   PDF/UA status: " + plop.pcos_get_string(doc, "pdfua"));
            Console.WriteLine("   PDF/VT status: " + plop.pcos_get_string(doc, "pdfvt"));

            Console.WriteLine("        XFA data: "
              + (plop.pcos_get_number(doc, "type:/Root/AcroForm/XFA") != 0 ? "yes" : "no"));

            Console.WriteLine("      Tagged PDF: " +
                ((plop.pcos_get_number(doc, "tagged") != 0) ? "yes" : "no"));

            /* --------- analyze digital signatures  */

            count = (int) plop.pcos_get_number(doc, "length:signaturefields");

            for (i = 0; i < count; i++){
                Console.Write("   Sig. field #" + (i+1) + ": '" + plop.pcos_get_string(doc, "signaturefields[" + i + "]/fullname") + "',");

                Console.Write((plop.pcos_get_number(doc, "signaturefields[" + i + "]/visible") != 0) ? " visible" : " invisible");
                
                
                sigtype = plop.pcos_get_string(doc, "signaturefields[" + i + "]/sigtype");

                if (sigtype == "none"){
                    Console.WriteLine(", unsigned");
                    continue;
                }else{
                    Console.Write(" " + sigtype + " signature");

                    if (sigtype == "certification"){
                        Console.Write(" (" + plop.pcos_get_string(doc, "signaturefields[" + i + "]/permissions") + " allowed)");
                    }

                    if (plop.pcos_get_number(doc, "signaturefields[" + i + "]/cades")>0) {
                        Console.Write(", CAdES");
                    }
                }
                Console.WriteLine("");

            }

            if (count > 0) {
                Console.WriteLine("No. of revisions: " + plop.pcos_get_number(doc, "revisions") + "");
                Console.WriteLine("");
            }

            Console.WriteLine("  Reader-enabled: " + plop.pcos_get_string(doc, "usagerights"));


            Console.WriteLine("    No. of pages: " +
                (int) plop.pcos_get_number(doc, "length:pages"));

            Console.WriteLine("     Page 1 size: width=" +
                 (int) plop.pcos_get_number(doc, "pages[0]/width") + ", height=" +
                 (int) plop.pcos_get_number(doc, "pages[0]/height"));

            count = (int) plop.pcos_get_number(doc, "length:fonts");
            Console.WriteLine("    No. of fonts: " +  count);

            for (i=0; i < count; i++)
            {
                String fonts;

                fonts = "fonts[" + i + "]/embedded";
                if (plop.pcos_get_number(doc, fonts) != 0)
                    Console.Write("embedded ");
                else
                    Console.Write("unembedded ");

                fonts = "fonts[" + i + "]/type";
                Console.Write(plop.pcos_get_string(doc, fonts) + " font ");
                fonts = "fonts[" + i + "]/name";
                Console.WriteLine(plop.pcos_get_string(doc, fonts));
            }

            Console.WriteLine("");

            plainmetadata = (int) plop.pcos_get_number(doc, "encrypt/plainmetadata") != 0;

            if (pcosmode == "restricted" && !plainmetadata &&
                        ((int) plop.pcos_get_number(doc, "encrypt/nocopy")) != 0)
            {
                Console.WriteLine(
                    "Restricted mode: no more information available");
                return(0);
            }

            /* ----- document info keys and XMP metadata (requires master pw) */

            count = (int) plop.pcos_get_number(doc, "length:/Info");

            for (i=0; i < count; i++)
            {
                String info;
                String key;
                int len;

                info = "type:/Info[" + i + "]";
                objtype = plop.pcos_get_string(doc, info);

                info = "/Info[" + i + "].key";
                key = plop.pcos_get_string(doc, info);
                len = 16 - key.Length;
                while (len-- > 0) Console.Write(" ");

                Console.Write(key + ": ");

                /* Info entries can be stored as string or name objects */
                if (objtype.Equals("name") || objtype.Equals("string"))
                {
                    info = "/Info[" + i + "]";
                    Console.WriteLine("'" + plop.pcos_get_string(doc, info) + "'");
                }
                else
                {
                    info = "type:/Info[" + i + "]";
                    Console.WriteLine("(" + plop.pcos_get_string(doc, info) +
                         " object)");
                }
            }

            Console.WriteLine();
            Console.Write("   XMP meta data: ");

            objtype = plop.pcos_get_string(doc, "type:/Root/Metadata");
            if (objtype.Equals("stream"))
            {
                byte[] contents;
                String ustring;

                contents = plop.pcos_get_stream(doc, "", "/Root/Metadata");
                // This demonstrates Unicode conversion 
                ustring = plop.convert_to_unicode("utf8", contents,
                                            "outputformat=utf16");
                Console.WriteLine("(" + ustring.Length + " Unicode code points");

                Console.WriteLine("");
            }
            else
            {
                Console.WriteLine("not present");
            }

            plop.close_document(doc, "");
        }

        catch (PLOPException e) {
            Console.WriteLine("Plop exception occurred in dumper:\n");
            Console.WriteLine("[" + e.get_errnum() + "] " + e.get_apiname() +
                            ": " + e.get_errmsg() + "\n");
        } catch (Exception e) {
            Console.WriteLine(e.Message);
        } finally {
            if (plop != null) {
                plop.Dispose();
            }
        }

        return(0);
    }
}
