/* 
 * PDFlib PLOP DS:
 * Demonstrate mass signature with optional PKCS#11 session handling
 */

// For testing PKCS#11 set the following define:
// #define USE_PKCS11

using System;
using PLOP_dotnet;


class Multisign
{
    static void Main(string[] args)
    {

        /* Basic signature options */
        const string sign_opts =
            "ltv=try " +         /* use ltv=full to ensure LTV status */
#if USE_PKCS11
            "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
#else
            "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} "
#endif
            ;

        // This is where the data files are. Adjust as necessary.
        const string searchpath_optlist = "searchpath={../data ../../data}";
        PLOP plop = null;

        if (args.Length < 1)
        {
#if NET40
            Console.WriteLine("usage: multisign <infile>...");
#else
            Console.WriteLine("usage: dotnet run <infile>...");
#endif
            return;
        }

        try
        {
            int doc;
            int i;
            string optlist;
            plop = new PLOP();

            plop.set_option(searchpath_optlist);

            if (plop.prepare_signature(sign_opts) == -1)
            {
                Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                return;
            }

            for (i = 0; i < args.Length; i++)
            {
                string signed_name;

                Console.WriteLine("Signing '{0}'...", args[i]);

                if ((doc = plop.open_document(args[i], "")) == -1)
                {
                    Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                    continue;
                }

                signed_name = args[i] + "-signed.pdf";
                optlist = "input=" + doc;
                if (plop.create_document(signed_name, optlist) == -1)
                {
                    Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());

#if USE_PKCS11
                    /* The PKCS#11 session may have expired;
                     * restart and try again.
                     */
                    if (plop.prepare_signature(sign_opts) == -1)
                    {
                        Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                        return;
                    }
                    if (plop.create_document(signed_name, optlist) == -1)
                    {
                        Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                        return;
                    }
#endif
                }

                plop.close_document(doc, "");
            }
        }
        catch (PLOPException e)
        {
            Console.WriteLine("PLOP exception occurred in multisign sample:");
            Console.WriteLine("[{0}] {1}: {2}", e.get_errnum(),
                e.get_apiname(), e.get_errmsg());
        }
        finally
        {
            if (plop != null)
            {
                plop.Dispose();
            }
        }
    }
}
