/* 
 * PDFlib PLOP DS:
 * Demonstrate digital signature features with optional signature visualization
 */

using System;
using PLOP_dotnet;

class Sign
{
    static void Main(string[] args) 
    {   
        /* Basic signature options */
        const string sign_opts =
            "ltv=try "  +        /* use ltv=full to ensure LTV status */
#if USE_PKCS11
            "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
#else
            "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} "
#endif
            ;

        // This is where the data files are. Adjust as necessary.
        const string searchpath_optlist = "searchpath={../data ../../data}";

        /* Set to true to enable signature visualization */
        Boolean visualize = false;
        const string visfile = "signing_man_pdfa1a.pdf";

        if (args.Length < 2)
        {
#if NET40
            Console.WriteLine("usage: sign <infile> <outfile>");
#else
            Console.WriteLine("usage: dotnet run <infile> <outfile>");
#endif
            return;
        }

        string input_file = args[0];
        string output_file = args[1];
        PLOP plop = null;


        try 
        {
            int doc, visdoc = -1;
            plop = new PLOP();

            plop.set_option(searchpath_optlist);

            if ((doc = plop.open_document(input_file, "")) == -1)
            {
                Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                return;
            }

            string optlist = sign_opts;

            if (visualize)
            {
                if ((visdoc = plop.open_document(visfile, "")) == -1)
                {
                    Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                    return;
                }

                /* Add field option for visualization */
                optlist += "field={visdoc=" + visdoc + " rect={50 50 adapt adapt}} ";
            }

            if (plop.prepare_signature(optlist) == -1)
            {
                Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                return;
            }

            if (plop.create_document(output_file, "input=" + doc) == -1)
            {
                Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                return;
            }

            plop.close_document(doc, "");

            if (visualize)
            {
                plop.close_document(visdoc, "");
            }
        }
        catch (PLOPException e) 
        {
            Console.WriteLine("PLOP exception occurred in sign sample:");
            Console.WriteLine("[{0}] {1}: {2}", e.get_errnum(),
                e.get_apiname(), e.get_errmsg());
        } finally {
            if (plop != null) {
                plop.Dispose();
            }
        }
    }
}
